<?php

// $Id: classes.inc,v 1.2 2009/11/25 23:34:54 nickurbits Exp $

/**
 * @file
 * Provides classes for module
 */
 
/**
 * Standard license object
 *
 * @param $key
 *  string - License key
 */
class license {
  // define properties
  private $key;
  public $lid;
  private $uid;
  private $shelf;
  
  // define methods
  /**
   * Construct method
   */
  function __construct($key_init) {
    if (is_numeric($key_init)) {
      // using lid to load
      $this->set_lid($key_init);
    }
    else {
      // using key to load
      $this->set_key($key_init);
    }
    $this->rebuild_license();
    licensing_invoke_all('licensingapi', 'load', $this);
  }
  
  /**
   * Method which returns the raw db data
   *
   * @param $type
   *  - 'key' loading from an actual license key
   *  - 'lid' loading from a license ID
   * @return $license
   *  raw license object from database
   */
  function get_license($type = 'key') {
    if ($type == 'key') {
      $sql = "SELECT * FROM {licensing_key} WHERE LOWER(license) = LOWER('%s') LIMIT 1";
      $arg = $this->get_key();
    }
    elseif ($type == 'lid') {
      $sql = "SELECT * FROM {licensing_key} WHERE lid = %d LIMIT 1";
      $arg = $this->get_lid();
    }
    
    // exit if function not used properly
    if (!$sql) return;
    
    return db_fetch_object(db_query($sql, $arg));
  }
  
  /**
   * Rebuilds current object from database based on current key
   */
  function rebuild_license() {
    if ($this->get_lid() && !$this->get_key()) {
      // load using lid
      $license = $this->get_license('lid');
    }
    else {
      // load using key
      $license = $this->get_license();
    }
    $this->_set_license($license);
  }
  
  /**
   * Private function which sets the properties from a license db object
   */
  private function _set_license($license) {
    if (isset($license->lid)) {
      $this->set_lid($license->lid);
      $this->set_uid($license->uid);
      $this->set_shelf($license->shelf);
      $this->set_key($license->license, FALSE); // we put false here to stop a rebuild loop
      if ($this->uid) {
        $this->set_shelf(1);
      }
    }
    else {
      $this->set_lid(0);
    }
  }
  
  /**
   * Following methods set properties
   */
  function set_key($key_new, $rebuild = TRUE) {
    $this->key = strtoupper($key_new);
    // if this license is set to an existing license, replace the whole object
    if ($this->get_license() && $rebuild) {
      $this->rebuild_license();
    }
  }
  
  function set_lid($lid_new) {
    $this->lid = $lid_new;
  }
  
  function set_uid($uid_new) {
    if (is_numeric($uid_new)) {
      $this->uid = $uid_new;
    }
  }
  
  function set_shelf($shelf_new) {
    $this->shelf = ($shelf_new == 1) ? TRUE : FALSE;
  }
  
  /**
   * Following methods retrieve properties
   */
  function get_key() {
    return $this->key;
  }
  
  function get_lid() {
    return $this->lid;
  }
  
  function get_uid() {
    return $this->uid;
  }
  
  function get_shelf() {
    return $this->shelf;
  }
  
  /**
   * Deduces if license key is valid
   *
   * @return TRUE if valid
   */
  function is_valid() {
    return ($this->get_lid()) ? TRUE : FALSE;
  }
  
  /**
   * Deduces if license key has ben registered
   *
   * @return TRUE if license key is registered
   */
  function is_registered() {
    return ($this->get_uid() > 0) ? TRUE : FALSE;
  }
  
  /**
   * Deduces if license key is available to be registered. Basically an inverse method
   * to is_registered
   *
   * @return TRUE if license key is not registered
   */
   function is_available() {
     return ($this->is_registered()) ? FALSE : TRUE;
   }
   
   /**
    * Deduces if the license key is available to be put on the shelf
    */
  function is_shelved() {
    return ($this->is_available() && $this->get_shelf()) ? TRUE : FALSE;
  }
  
  /**
   * returns user object of registered user
   * results are statically cached.
   */
  function get_user() {
    static $users = array();
    if ($users[$this->get_uid()]) {
      return $users[$this->get_uid()];
    }
    $users[$this->get_uid()] = user_load($this->get_uid());
    return $users[$this->get_uid()];
  }
   
   /**
    * Saves the license object to the database in it's current state
    */
  function save() {
	licensing_invoke_all('licensingapi', 'presave', $this);
    db_query($this->_save_prepare_sql($this->get_lid()), $this->_save_prepare_data($this->get_lid()));
    $this->rebuild_license();
    licensing_invoke_all('licensingapi', 'postsave', $form['#license'], $form_state);
  }
  
  /**
   * Protected method which builds SQL for saving data
   *
   * @param $update
   *  - 1 if running UPDATE statement
   *  - 0 if running INSERT statement
   * @return $sql
   *  string sql query
   */
  protected function _save_prepare_sql($update = 0) {
    if ($update) {
      // object already exists in the database, update
      $sql = "UPDATE {licensing_key} SET license = '%s', uid = %d, shelf = %d WHERE lid = %d";
    }
    else {
      // create new entry in the database
      $sql = "INSERT INTO {licensing_key} (license, uid, shelf) VALUES ('%s', %d, %d)";
    }
    return $sql;
  }
  
  /**
   * Protected method which prepares data for saving
   *
   * @param $update
   *  - 1 if running UPDATE statement
   *  - 0 if running INSERT statement
   * @return $args
   *  associative array of arguments to pass into SQL
   */
  protected function _save_prepare_data($update = 0) {
    $args = array(
      $this->get_key(),
      ($this->get_uid()) ? $this->get_uid() : 0,
      ($this->get_shelf()) ? 1 : 0,
    );
    if ($update) {
      $args[] = $this->get_lid();
    }
    return $args;
  }
  
  /**
   * method which deletes the current license from the database
   */
  function delete() {
    $sql = "DELETE FROM {licensing_key} WHERE lid = %d LIMIT 1;";
    db_query($sql, $this->get_lid());
  }
  
  /**
   * Debug method. print_r's the object to the screen
   */
  function debug($debug_func = NULL) {
    if (!$debug_func) {
      $debug_func = (module_exists('devel')) ? 'dpm' : 'print_r';
    }
    call_user_func_array($debug_func, array(array('Object Public Properties' => $this)));
    call_user_func_array($debug_func, array(array('Object Methods' => get_class_methods('license'))));
  }
}






