<?php

// $Id: form.inc,v 1.2 2009/11/25 23:34:54 nickurbits Exp $

/**
 * @file 
 *  contains all form building & handling
 */

/**
 * Form generation for manual creation form.
 */
function licensing_manual_form() {
  $form['qty'] = array(
    '#type' => 'textfield',
    '#title' => t('Quantity'),
    '#description' => t('Select how many licenses you wish to create'),
    '#required' => TRUE,
    '#size' => '4',
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Generate licenses!'),
  );
  return $form;
}

/**
 * Validation for manual creation form.
 */
function licensing_manual_form_validate($form, &$form_state) {
  $trouble = is_numeric($form_state['values']['qty']);
  if (!$trouble) {
    $problem = 'chars';
  }
  if ($form_state['values']['qty'] >= 1001) {
    $problem = 'value';
  }
  if ($problem) {
    $output[] = 'This is not a valid quantity.';
    switch ($problem) {
    case 'chars' :
      $output[] = 'Please omit any non-numeric characters.';
      break;
    case 'value' :
      $output[] = 'Please keep the quantity at 1000 or less.';
      break;
    }
    form_set_error('qty', t(implode(' ', $output)));
  }
}

/**
 * Submit handler for manual creation form.
 */
function licensing_manual_form_submit($form, &$form_state) {
  // generates keys
  $keys = licensing_generate_key($form_state['values']['qty']);
  drupal_set_message(theme('licensing_generate_success_message', $keys));
}

/**
 * Form generation for key verification form.
 */
function licensing_verify_form() {
  $form['key'] = array(
    '#type' => 'textarea',
    '#title' => t('License key(s)'),
    '#description' => t('Copy <em>license key(s)</em> here, and seperate with a comma then space (, ).'),
    '#required' => TRUE,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Verify',
  );
  return $form;
}

/**
 * Submit handler for key verification form.
 */
function licensing_verify_form_submit($form, &$form_state) {
  $keys = explode(', ', $form_state['values']['key']);
  foreach ($keys as $key) {
    $licenses[] = new license($key);
  }
  drupal_set_message(theme('licensing_verify_results', $licenses));
}

/**
 * Form generation for key export form.
 */
function licensing_export_form() {
  $form['qty'] = array(
	  '#type' => 'textfield',
		'#title' => 'Quantity',
		'#description' => 'Amount of license keys you wish to export',
		'#required' => TRUE,
		'#size' => '6',
		'#maxlength' => '4',
	);
	$form['type'] = array(
	  '#type' => 'radios',
		'#title' => 'Export format',
		'#description' => 'Way in which you want to export the license keys',
		'#options' => array(
		  'csv' => 'Comma-Separated Values (.csv)', 
			'display' => 'Display on the page',
		),
		'#required' => 1,
	);
	$form['shelf'] = array(
	  '#type' => 'checkbox',
		'#title' => 'Mark as shelf-ready?',
		'#description' => '<strong>Only use this if you intend to distribute the key</strong>. This will mark all the keys that are exported, needing them to be unmarked if you wish to re-print them.',
		'#required' => 0,
    '#default_value' => 1,
	);
	$form['submit'] = array(
	  '#type' => 'submit',
		'#value' => 'Export!',
  );
	return $form;
}

/**
 * Validation for key export form.
 */
function licensing_export_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['qty'])) {
	  form_set_error('qty', t('<span style="font-weight: bold; color:red;">You entered an invalid quantity, please re-enter it.</span>'));
	}
}

/**
 * Submit handler for key export form.
 */
function licensing_export_form_submit($form, &$form_state) {
  // get keys from database, shelf if required
  $keys = licensing_retrieve_key($form_state['values']['qty']);
  if ($form_state['values']['shelf']) {
    licensing_shelf_key($keys);
  }
  switch ($form_state['values']['type']) {
  case 'display' :
    // display to screen
    drupal_set_message(theme('licensing_export_screen', $keys, $form_state['values']['shelf']));
  break;  
  case 'csv' :
    if ($export = licensing_export_csv($keys)) {
      drupal_set_message(theme('licensing_export_csv_message', $export['filename'], $export['filepath_public'], $form_state['values']['shelf']));
    }
    else {
      drupal_set_message('Failed to export CSV file', 'error');
    }
  break;
  }
  drupal_set_message($output);
}

/**
 * Form builder for license key edit form
 */
function licensing_edit_license() {
  if (!is_numeric(arg(4))) { return array(); }
  $license = new license(arg(4));
  $form = array(
    '#license' => $license,
    'lid' => array(
      '#title' => t('License ID'),
      '#description' => t('Unique identifier for this key, can not be edited.'),
      '#type' => 'textfield',
      '#disabled' => TRUE,
      '#value' => $license->get_lid(),
    ),
    'key' => array(
      '#title' => t('License Key'),
      '#description' => t('Generally, the actual license key should not be modified. Ensure it is unique if you find yourself having to do so.'),
      '#type' => 'textfield',
      '#default_value' => $license->get_key(),
      '#disabled' => (!$license->is_shelved()) ? FALSE : TRUE,
    ),
    'shelf' => array(
      '#title' => t('Shelf status'),
      '#description' => t('Whether the key is available to be put on the shelf. <sup>Disabled when a user has registered this key</sup>'),
      '#type' => 'checkbox',
      '#default_value' => $license->get_shelf(),
      '#disabled' => $license->is_registered(),
    ),
    'uid' => array(
      '#title' => t('Registered user'),
      '#description' => t('The user who has registered this license key'),
      '#type' => 'textfield',
      '#autocomplete_path' => 'user/autocomplete',
      '#default_value' => ($license->get_uid()) ? user_load($license->get_uid())->name : NULL,
    ),
    'buttons' => array(
      'save' => array(
        '#type' => 'submit',
        '#value' => 'Save!',
      ),
      'delete' => array(
        '#type' => 'submit',
        '#value' => 'Delete!',
        '#submit' => array(
          'licensing_edit_license_delete',
        ),
      ),
    ),
    '#redirect' => array('admin/settings/licensing/key'),
  );
  return $form;
}

function licensing_edit_license_delete(&$form, &$form_state) {
  $destination = '';
  if (isset($_REQUEST['destination'])) {
    $destination = drupal_get_destination();
    unset($_REQUEST['destination']);
  }
  $license = $form['#license'];
  $form['#redirect'] = array('admin/settings/licensing/key/'. $license->get_lid() .'/delete', $destination);
}

function licensing_edit_license_submit($form, $form_state) {
  $license = new license($form_state['values']['lid']);
  
  if ($form_state['values']['key'] && ($form_state['values']['key'] != $license->get_key())) {
    // if key exists in form state and is different to current licenes
    $license->set_key($form_state['values']['key']);
  }
  if ($form_state['values']['shelf'] != $license->get_shelf()) {
    // if shelf status dont match
    $license->set_shelf($form_state['values']['shelf']);
  }
  if ($form_state['values']['uid']) {
    $sql = "SELECT uid FROM {users} WHERE name = '%s' LIMIT 1;";
    $uid = db_result(db_query($sql, $form_state['values']['uid']));
    if ($form_state['values']['uid'] != $license->get_uid()) {
      // users dont match
      $license->set_uid($uid);
    }
  }
  if (!$form_state['values']['uid'] && $license->get_uid()) {
    // user has been removed
    $license->set_uid(0);
  }
  $license->save();
  drupal_set_message('Your changes have been saved!');
}

/**
 * Form builder for license delete page
 */
function licensing_form_delete() {
  return array(
    '#redirect' => array('admin/settings/licensing/key'),
    '#license' => new license(arg(4)),
    'buttons' => array(
      'submit' => array(
        '#type' => 'submit',
        '#value' => 'Delete!',
        '#submit' => array(
          'licensing_delete_delete_submit',
        ),
      ),
      'cancel' => array(
        '#type' => 'submit',
        '#value' => 'Cancel',
      ),
    ),
  );
}

/**
 * Validation handler for license delete page
 */
function licensing_delete_validate(&$form, &$form_state) {
  if (isset($_REQUEST['destination'])) {
    $destination = $_REQUEST['destination'];
  }
  else {
    $destination = 'admin/settings/licensing/key';
  }
  $form_state['#redirect'] = array($destination, '');
}

/**
 * submit handler for actually deleting on license delete page
 */
function licensing_delete_delete_submit($form, $form_state) {
  $license = $form['#license'];
  $license->delete();
  drupal_set_message(t('The license key '. $license->get_key() .' has been deleted'));
}

/**
 * Form builder for view page
 */
function licensing_form_view() {
  return array(
    'limit' => array(
      '#type' => 'textfield',
      '#size' => 5,
      '#title' => t('Maximum results to return'),
      '#default_value' => (is_numeric($_GET['limit'])) ? $_GET['limit'] : 30,
    ),
    'start' => array(
      '#type' => 'textfield',
      '#size' => 5,
      '#title' => t('Result to begin from'),
      '#default_value' => (is_numeric($_GET['start'])) ? $_GET['start'] : 0,
    ),
    'submit' => array(
      '#type' => 'submit',
      '#value' => 'Submit',
    ),
    '#method' => 'GET',
  );
}

/**
 * Form builder for license key registration page. We have this as a seperate function so
 * it is still passed through hook_form_alter
 */
function licensing_form_register() {
  return licensing_invoke_all('licensing_register', 'form');
}


/**
 * Form validation function for registration page
 */
function licensing_form_register_validate(&$form, &$form_state) {
  $form['#license'] = new license($form_state['values']['license']);
  // invoke hook_licensing_register
  licensing_invoke_all('licensing_register', 'validate', $form['#license'], $form_state['values'], str_replace('-', '_', $form['#id']));
}

/**
 * Form submission function for registration page
 */
function licensing_form_register_submit(&$form, &$form_state) {
  $form['#license'] = new license($form_state['values']['license']);
  // invoke hook_licensing_register
  licensing_invoke_all('licensing_register', 'submit', $form['#license'], $form_state['values'], str_replace('-', '_', $form['#id']));
}