<?php

// $Id: theme.inc,v 1.2 2009/11/25 23:34:54 nickurbits Exp $

/**
 * @file
 *  Contains theme functions specific to license module
 */

/**
 * Theme function for styling confirmation message when creating license keys
 *
 * @param $keys
 *  associative array of lid => key
 * @return
 *  Themed HTML
 */
function theme_licensing_generate_success_message($keys) {
  $output = '<h3 class="title">The following keys were generated:</h3>';
  $output .= '<ul class="license-keys">';
  foreach ($keys as $lid => $key) {
    $output .= '<li>'. l($key, 'admin/settings/mcr/license/key/'. $lid) .'</li>';
  }
  $output .= '</ul>';
  return $output;
}

/**
 * Themes export message headers
 *
 * @param $shelved
 *  TRUE if keys are shelved
 * @return
 *  themed HTML
 */
function theme_licensing_export_message_header($shelved = TRUE) {
  $message = t('You have successfully exported some license keys');
  $message .= ($shelved) ? t(' and marked them as <em>shelf ready</em>') : '';
  $message .= ':';
  return $message;
}

/**
 * Theme function for exporting license key(s) to screen
 *
 * @param $keys
 *  associative array of lid => key
 * @param $shelved
 *  TRUE if keys have been marked shelf ready
 * @return $message
 *  Themed HTML
 */
function theme_licensing_export_screen($keys = array(), $shelved = TRUE) {
  $message = theme('licensing_export_message_header', $shelved);
    
  // make each key a link to it's edit page
  foreach ($keys as $lid => $key) {
    $key_links[$lid] = l($key, 'admin/settings/mcr/license/'. $lid);
  }
  
  $message .= '<ul><li>'. implode('</li><li>', $key_links) .'</li></ul>';
  return $message;
}

/**
 * Theme function for export to csv confirmation message
 *
 * @param $csvname
 * @param $csvpath
 *  path to exported csv file
 * @param $shelved
 *  TRUE if key has been shelved
 * @return
 *  Themed HTML
 */
function theme_licensing_export_csv_message($csvname = NULL, $csvpath = NULL, $shelved = TRUE) {
  $message = theme('licensing_export_message_header', $shelved);
  $message .= '<br />'. l(t('Download '). $csvname, $csvpath);
  $message .= '<br /><sup><em>'. t('Right click > Save as') .'</em> | '. t('This is the CSV file containing the requested amount of valid license keys') .'</sup>';
  return $message;
//  $output .= '<a href="/sites/default/files/'. $filename .'">Download '. $filename .'</a>.<br><span style="font-size:10px;"><strong>Right click > Save As</strong>. This is the CSV file containing the requested amount of valid license keys.</span>';
}

/**
 * Theme function for verification results message
 *
 * @param $licenses 
 *  array of license objects
 * @return
 *  themed HTML
 */
function theme_licensing_verify_results($licenses = array()) {
  $headers = array(
    'Key',
    'Valid',
    'Shelf status',
    'Available',
    'Operations',
  );
  foreach ($licenses as $key => $license) {
    $rows[$key] = array(
      array(
        'data' => $license->get_key(),
        'style' => 'font-weight: strong; color: black;',
      ),
      array(
        'data' => ($license->is_valid()) ? 'Valid' : 'Invalid',
        'style' => ($license->is_valid()) ? 'color: green;' : 'color: red;',
      ),
    );
    if ($license->is_valid()) {
      $rows[$key] = array_merge(
        $rows[$key], 
        array(
          array(
            'data' => ($license->get_shelf()) ? 'On shelf': 'In stockroom',
            'style' => ($license->get_shelf()) ? 'color: black;' : 'color: grey;',
          ),
          array(
            'data' => ($license->is_available()) ? 'Available' : 'Registered',
            'style' => ($license->is_available()) ? 'color: green;' : 'color: orange;',
          ),
          array(
            'data' => l('edit', 'admin/settings/mcr/license/'. $license->get_lid()),
          ),
        )
      );
    }
  }
  return theme('table', $headers, $rows);
}

/**
 * Themes the list of license keys registered to a user on the user page
 *
 * @param $variables
 *  array(
 *    #user => user object
 *    #licenses => license objects
 *  )
 * @return
 *  themed content
 */
function theme_licensing_user_view_licenses($variables = array()) {
  $title = '<h3>Licensing information</h3>';
  $infos = array();
  foreach ($variables['#licenses'] as $key => $license) { 
    $new_info = licensing_invoke_all('licensingapi', 'userview', $license);
    if (is_array($new_info)) {
      $infos = array_merge($infos, array($new_info));
    }
  }
  foreach ($infos as $info) {
    foreach ($info as $column_name => $values) {
      $header[$column_name]['data'] = $values['title'];
      foreach ($values['data'] as $lid => $value) {
        $rows[$lid][$column_name]['data'] = $value;
      }
      unset($values['data']);
      if (is_array($values)) {
        foreach ($values as $param => $value) {
          $rows[$lid][$column_name][$param] = $value;
        }
      }
    }
  }
  return (!empty($rows)) ? $title . theme('table', $header, $rows) : NULL;
}